"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const inquirer = require("inquirer");
const locators_1 = require("./helpers/locators");
const pageHelper_1 = require("./helpers/pageHelper");
const config_1 = require("./utils/config");
const config = require('../resources/config.json');
const wishes = [
    'Happy Birthday!',
    'Happy Birthday Buddy!',
    'Happy Birthday my Friend!',
    'Wish you many many happy returns of the day!',
    'I want to write my own custom message!',
];
exports.wishes = wishes;
class BirthdayWisher {
    constructor() {
        this.page = new pageHelper_1.PageHelper();
    }
    /**
     * @param  {string[]} names
     */
    birthdayQuestions(names) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield inquirer.prompt([
                {
                    choices: names,
                    message: 'Please select the people you wanna wish?',
                    name: 'birthdayName',
                    type: 'list',
                },
                {
                    choices: wishes,
                    message: 'Select your favourite birthday wish:',
                    name: 'wish',
                    type: 'list',
                },
                {
                    message: 'Please write your own custom wish:',
                    name: 'customWish',
                    type: 'input',
                    when: (birthdayAnswers) => birthdayAnswers.wish.indexOf('custom message') > -1,
                },
            ]);
        });
    }
    /**
     * @param  {any} answers
     */
    login(answers) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.page.init();
                yield this.page.open(locators_1.locators.url);
                yield this.page.focusElement(locators_1.locators.username);
                yield this.page.clearElement(locators_1.locators.username);
                yield this.page.sendElementText(locators_1.locators.username, answers.username);
                yield this.page.focusElement(locators_1.locators.password);
                yield this.page.clearElement(locators_1.locators.password);
                yield this.page.sendElementText(locators_1.locators.password, answers.password);
                yield Promise.all([
                    this.page.clickElement(locators_1.locators.loginButton),
                    this.page.waitForNavigation({ waitUntil: 'networkidle2' }),
                ]);
            }
            catch (Exception) {
                try {
                    config.firstLogin = true;
                    config.username = '';
                    config.password = '';
                    yield config_1.writeFile(config);
                    const title = yield this.page.getTitle();
                    if (title.indexOf('Events') === -1) {
                        throw new Error('Login Failed! Most likely due to Invalid Credentials or below reason :' +
                            '\n' + Exception.toString());
                    }
                }
                catch (Exception) {
                    throw Exception;
                }
            }
        });
    }
    /**
     * @returns Promise<string[]>
     */
    fetchBirthdayNames() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.page.getText(locators_1.locators.birthdayNames);
        });
    }
    /**
     * @returns Promise<ElementHandle[]>
     */
    fetchAllTexts() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.page.$$(locators_1.locators.birthdayText);
        });
    }
    /**
     * @param  {string[]} birthdayNames
     * @param  {ElementHandle[]} birthdayTexts
     */
    wishAll(birthdayNames, birthdayTexts, wish) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.page.enterAllElementText(birthdayTexts, wish);
            console.log(chalk_1.default.green('\nSuccessfully wished:\n'));
            birthdayNames.forEach((name) => {
                console.log(chalk_1.default.green(name + '\n'));
            });
            config.birthdayNames = [];
            yield config_1.writeFile(config);
        });
    }
    /**
     * @param  {string[]} birthdayNames
     * @param  {ElementHandle[]} birthdayTexts
     */
    birthdayWish(credentials, birthdayNames, birthdayTexts) {
        return __awaiter(this, void 0, void 0, function* () {
            let indexOfBirthdayName;
            const birthdayAnswers = yield this.birthdayQuestions(birthdayNames);
            indexOfBirthdayName = birthdayNames.indexOf(birthdayAnswers.birthdayName);
            if (birthdayAnswers.wish.indexOf('custom message') > 0) {
                yield this.page.enterElementText(indexOfBirthdayName, birthdayTexts, birthdayAnswers.customWish);
            }
            else {
                yield this.page.enterElementText(indexOfBirthdayName, birthdayTexts, birthdayAnswers.wish);
            }
            console.log(chalk_1.default.green('\nSuccessfully wished ' + birthdayAnswers.birthdayName) + '\n');
            birthdayNames.splice(indexOfBirthdayName, 1);
            config.birthdayNames = birthdayNames;
            yield config_1.writeFile(config);
            if (birthdayNames.length > 0) {
                const moreWishes = yield inquirer.prompt([
                    {
                        default: true,
                        message: 'Do you want to wish more of your friends?',
                        name: 'friends',
                        type: 'confirm',
                    },
                ]);
                if (moreWishes.friends) {
                    yield this.birthdayWish(credentials, birthdayNames, birthdayTexts);
                }
                else {
                    yield config_1.encryptCredentials(credentials);
                    config.firstLogin = false;
                    yield config_1.writeFile(config);
                }
            }
        });
    }
    /**
     * @param  {any} credentials
     */
    findAndWish(credentials) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (yield this.page.isElementExists(locators_1.locators.birthdayTodayCard)) {
                    const today = new Date().getDay();
                    const birthdayTexts = yield this.fetchAllTexts();
                    if (!config.firstLogin) {
                        if (config.day === today) {
                            if (config.birthdayNames.length > 0) {
                                yield this.birthdayWish(credentials, config.birthdayNames, birthdayTexts);
                            }
                            else if (config.birthdayNames.length === 0) {
                                // move this cli.ts, not first time login , skip login part!
                                throw new Error('You have wished all your friends, Please try tomorrow!');
                            }
                        }
                        else {
                            config.day = today;
                            config.birthdayNames = yield this.fetchBirthdayNames();
                            yield config_1.writeFile(config);
                            yield this.birthdayWish(credentials, config.birthdayNames, birthdayTexts);
                        }
                    }
                    else {
                        config.birthdayNames = yield this.fetchBirthdayNames();
                        yield config_1.writeFile(config);
                        yield this.birthdayWish(credentials, config.birthdayNames, birthdayTexts);
                    }
                }
                else {
                    console.error('\n' + chalk_1.default.red('Uh oh, looks like none of your friends have birthdays today!\n'));
                }
                yield this.page.logout();
                process.exit(0);
            }
            catch (Exception) {
                console.error(chalk_1.default.red('\n' + Exception.toString() + '\n'));
                try {
                    yield this.page.logout();
                    process.exit(0);
                }
                catch (Exception) {
                    throw Exception;
                }
            }
        });
    }
    /**
     * @param  {any} credentials
     * @param  {string} wish
     */
    findAndWishAll(credentials, wish) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (yield this.page.isElementExists(locators_1.locators.birthdayTodayCard)) {
                    const today = new Date().getDay();
                    const birthdayTexts = yield this.fetchAllTexts();
                    if (!config.firstLogin) {
                        if (config.day === today) {
                            if (config.birthdayNames.length > 0) {
                                yield this.wishAll(config.birthdayNames, birthdayTexts, wish);
                            }
                            else if (config.birthdayNames.length === 0) {
                                throw new Error('You have wished all your friends, Please try tomorrow!');
                            }
                        }
                        else {
                            config.day = today;
                            config.birthdayNames = yield this.fetchBirthdayNames();
                            yield config_1.writeFile(config);
                            yield this.wishAll(config.birthdayNames, birthdayTexts, wish);
                        }
                    }
                    else {
                        config.birthdayNames = yield this.fetchBirthdayNames();
                        yield config_1.writeFile(config);
                        yield this.wishAll(config.birthdayNames, birthdayTexts, wish);
                        yield config_1.encryptCredentials(credentials);
                        config.firstLogin = false;
                        yield config_1.writeFile(config);
                    }
                }
                else {
                    console.error('\n' + chalk_1.default.red('Uh oh, looks like none of your friends have birthdays today!\n'));
                }
                yield this.page.logout();
                process.exit(0);
            }
            catch (Exception) {
                console.error(chalk_1.default.red('\n' + Exception.toString() + '\n'));
                try {
                    yield this.page.logout();
                    process.exit(0);
                }
                catch (Exception) {
                    throw Exception;
                }
            }
        });
    }
}
exports.BirthdayWisher = BirthdayWisher;
//# sourceMappingURL=birthday.js.map