import { ElementHandle, Response } from 'puppeteer';
declare class PageHelper {
    private browser;
    private page;
    private readonly retryCount;
    constructor();
    init(): Promise<void>;
    /**
     * @param  {string} url
     * @returns Promise
     */
    open(url: string): Promise<Response>;
    /**
     * @returns Promise
     */
    getTitle(): Promise<string>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    $$(element: string): Promise<ElementHandle[]>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    focusElement(element: string): Promise<void>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    clickElement(element: string): Promise<void>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    xpathClick(element: string): Promise<void>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    clearElement(element: string): Promise<void>;
    /**
     * @param  {string} element
     * @param  {string} text
     * @returns Promise
     */
    sendElementText(element: string, text: string): Promise<void>;
    /**
     * @param  {string} keys
     * @returns Promise
     */
    enterKeys(keys: string): Promise<void>;
    /**
     * @param  {ElementHandle} element
     * @param  {any} answers
     * @returns Promise
     */
    enterText(element: ElementHandle, text: string): Promise<void>;
    /**
     * @param  {ElementHandle[]} elements
     * @param  {any} answers
     * @returns Promise
     */
    enterAllElementText(elements: ElementHandle[], text: string): Promise<any>;
    /**
     * @param  {number} index
     * @param  {ElementHandle[]} elements
     * @param  {string} text
     * @returns Promise
     */
    enterElementText(index: number, elements: ElementHandle[], text: string): Promise<void>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    getText(element: string): Promise<string[]>;
    /**
     * @param  {any} options?
     * @returns Promise
     */
    waitForNavigation(options?: any): Promise<Response>;
    /**
     * @param  {string} element
     * @returns Promise
     */
    isElementExists(element: string): Promise<boolean>;
    /**
     * @returns Promise
     */
    logout(): Promise<void>;
}
export { PageHelper };
